import os
import urllib
import HTMLParser
import re

import gtk
import pango

import YahooDict
import DictResult
from Commons import ConnectException

del_js_p = re.compile("\'([^']+)\'")

class DictResultViewer(HTMLParser.HTMLParser, gtk.TextView):
    DEFAULT_CURSOR = gtk.gdk.Cursor(gtk.gdk.LEFT_PTR)
    LINK_CURSOR = gtk.gdk.Cursor(gtk.gdk.HAND2)
    WAIT_CURSOR = gtk.gdk.Cursor(gtk.gdk.WATCH)
    
    def __init__(self):
        HTMLParser.HTMLParser.__init__(self)
        gtk.TextView.__init__(self)

        self.set_editable(False)
        self.set_pixels_above_lines(3)
        self.set_left_margin(12)
        self.set_right_margin(12)
        self.set_wrap_mode(gtk.WRAP_WORD)
        self.buffer = gtk.TextBuffer()
        self.set_buffer(self.buffer)
        
        self.font_tree = []
        self.link = ''
        self.on_b = False
        self.bold = self.buffer.create_tag()
        self.bold.set_property('weight',700)
        self.hovering_over_link = False
        self.header = None

        self.dict = YahooDict.YahooDict()

        self.connect("event_after", self.on_event_after)
        self.connect("motion_notify_event", self.on_motion_notify_event)
        self.connect("visibility_notify_event" , self.on_visibility_notify_event)

    def set_header(self, header):
        self.header = header
    
    def search(self, word, lang):
        self._show_result(self.dict.search(word, lang))
    
    def _clear(self):
        start, end = self.buffer.get_bounds()
        self.buffer.delete(start, end)
    
    def _show_result(self, result):
        html_data = result.getResult()
        html_data = unicode(html_data, 'euc-kr','ignore')
        self._clear()
        if self.header:
            it = self.buffer.get_start_iter()
            anchor = self.buffer.create_child_anchor(it)
            self.add_child_at_anchor(self.header, anchor)
            self.header.show_all()
        try:
            self.feed(html_data)
            self.close()
        except HTMLParser.HTMLParseError:
            pass

    def _download(self,url):
        name = os.path.join(os.path.join(os.getenv('HOME'), '.gdick'), os.path.basename(url))
        if not os.path.exists(name):
            f = open(name,'w')
            f.write(urllib.urlopen(url).read())
            f.close()
            os.system("convert -draw 'matte 0,0 replace' %s %s" % (name, name))
        pixbuf = gtk.gdk.pixbuf_new_from_file(name)
        end = self.buffer.get_end_iter()
        self.buffer.insert_pixbuf(end, pixbuf)

    def _cursor_change(self, cursor, widget = None):
        if not widget:
            widget = self.win.window
        widget.set_cursor(cursor)

    def _set_cursor_if_appropriate(self, tv, x, y):
        hovering = False
        buffer = tv.get_buffer()
        for tag in tv.get_iter_at_location(x,y).get_tags():
            url = tag.get_data("url")
            if url and url != "":
                hovering = True
                break
        if hovering != self.hovering_over_link:
            self.hovering_over_link = hovering
            if self.hovering_over_link:
                self.hovering_over_link_tag = tag
                self.hovering_over_link_tag.set_property("underline", pango.UNDERLINE_SINGLE)
                self.hovering_over_link_tag.set_property("foreground", "red")
                self._cursor_change(self.LINK_CURSOR, tv.get_window(gtk.TEXT_WINDOW_TEXT))
            else:
                self.hovering_over_link_tag.set_property("underline", pango.UNDERLINE_NONE)
                self.hovering_over_link_tag.set_property("foreground-gdk", tv.get_default_attributes().fg_color)
                self._cursor_change(self.DEFAULT_CURSOR, tv.get_window(gtk.TEXT_WINDOW_TEXT))
    
    # textview signal methods
    def on_motion_notify_event(self, textview, event):
        x, y = textview.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET, int(event.x), int(event.y))
        self._set_cursor_if_appropriate(textview, x, y)
        textview.window.get_pointer()

    def on_visibility_notify_event(self, textview, event):
        x, y, m = textview.window.get_pointer()
        bx, by = textview.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET, x, y)
        self._set_cursor_if_appropriate(textview, bx, by)
    
    def on_event_after(self, textview, event):
        if (event.type != gtk.gdk.BUTTON_RELEASE or event.button != 1):
            return gtk.FALSE
        buffer = textview.get_buffer()
        x, y = textview.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET, int(event.x), int(event.y))
        iter = textview.get_iter_at_location(x,y)
        for tag in iter.get_tags():
            data = tag.get_data("url")
            if data:
                if not data[-1].isdigit():
                    self.search(data)
                else:
                    url = "http://msrv.yahoo.co.kr/engdic/sound/"
                    url += data[0]
                    url += "/" + data + ".wma"
                    if self.speaker == 'browser':
                        webbrowser.open_new(url)
                    else:
                        os.system("mplayer " + url)
            break
    
    # HTMLParse methods
    def handle_starttag(self, name, attrs):
        if name == 'font':
            tag = self.buffer.create_tag()
            for attr, value in attrs:
                if attr == 'color':
                    tag.set_property('foreground',value)
                    break
            self.font_tree.append(tag)
        elif name == 'b':
            self.on_b = True
        else:
            for attr, value in attrs:
                if attr == 'href':
                    m = del_js_p.findall(value)
                    if m:
                        self.link = m[-1]
                    break
                elif attr == 'src':
                    self._download(value)
                    break

    def handle_endtag(self, tag):
        if tag == 'a':
            self.link = ''
        elif tag == 'font':
            try:
                del self.font_tree[-1]
            except IndexError: # malformed html :(
                pass
        elif tag == 'b':
            self.on_b = False

    def handle_data(self, data):
        end = self.buffer.get_end_iter()
        if self.link:
            tag = self.buffer.create_tag()
            tag.set_data("url", self.link)
            self.buffer.insert_with_tags(end, data, tag)
        elif self.font_tree != []:
            tag = self.font_tree[-1]
            self.buffer.insert_with_tags(end, data, tag)
        elif self.on_b:
            self.buffer.insert_with_tags(end, data, self.bold)
        else:
            self.buffer.insert(end, data)

    def handle_charref(self, name):
        end = self.buffer.get_end_iter()
        self.buffer.insert(end, unichr(int(name)))

    def handle_entityref(self, name):
        end = self.buffer.get_end_iter()
        if name == 'nbsp':
            self.buffer.insert(end, ' ')
