#!/usr/bin/env python
"""
gdick.py by ddt<ddt@mirstyle.org>
license: GPL
"""
import pygtk
pygtk.require("2.0")

import gtk
import gtk.glade
import gobject
import gconf

import os.path
import webbrowser

import DictResultViewer
from Commons import getUniqueList, sleeper, Worker

class GDick:
    def __init__(self):
        self.history = []
        self.lang = 0
        self.sameWordsShow = False

        h = os.getenv('HOME') + '/.gdick'
        if not os.path.exists(h):
            os.mkdir(h)
        
        self.conf = gconf.client_get_default()
        self.conf.add_dir("/apps/gdick", gconf.CLIENT_PRELOAD_NONE)
        self.conf.notify_add("/apps/gdick/speaker", self.on_gconf_speaker_key_changed)
        self.speaker = self.conf.get_string("/apps/gdick/speaker")
        if not self.speaker:
            self.conf.set_string("/apps/gdick/speaker", "mplayer")
        
        self.glade = gtk.glade.XML("/usr/share/gdick/gdick.glade")
        self.win = self.glade.get_widget('window')
        self.win.drag_dest_set(gtk.DEST_DEFAULT_ALL,
                               [('STRING',0,0),('text/plain',0,0)],
                               gtk.gdk.ACTION_COPY)
        self.combo = self.glade.get_widget('combo')
        self.textview = DictResultViewer.DictResultViewer()
        self.textview.show()
        
        box = self.glade.get_widget('scrolledwindow1')
        box.add(self.textview)
        
        # signal connect
        signals = {}
        for key in dir(self.__class__):
            if key[:3] == 'on_':
                signals[key] = getattr(self, key)
        self.glade.signal_autoconnect(signals)

    def status_update(self, msg):
        sb = self.glade.get_widget('statusbar')
        id = sb.get_context_id("gdick status")
        sb.pop(id)
        sb.push(id, msg)

    def search_wrap(self):
        word = self.combo.entry.get_text()
        self.search(word)
    
    def search(self, word):
        if not word or word == "":
            return
        word = word.strip()
	word = word.lower()
        self.history.insert(0,word)
        self.history = getUniqueList(self.history)
        self.history = self.history[:10]
        self.combo.set_popdown_strings(self.history)
        self.win.set_title(word + ' - gdick')
        
        if word.isalpha() or self.lang == 0:
            lang = 'en'
        else:
            lang = 'ko'
        if isinstance(word, unicode):
            word = word.encode('euc-kr')
        else:
            word = word.decode('utf-8').encode('euc-kr')
        #self.textview.search(word, lang)
        Worker(self.textview, word, lang).start()
        self.win.present()

    def main(self):
        gtk.main()
    
    ###################
    # signal handlers #
    ###################
    def on_gconf_speaker_key_changed(self, client, cnxn_id, entry, *args):
        self.speaker = entry.value.to_string()
    
    def on_window_size_request(self, widget, requisition, *args):
        if self.sameWordsShow:
            size = self.win.get_size()
            self.paned.set_position(size[0]-100)
            
    def on_treeview_row_activated(self, treeview, path, view_column, *args):
        model = treeview.get_model()
        iter = model.get_iter(path)
        self.search(model.get_value(iter, 0))
    
    def on_english_activate(self, widget):
        self.lang = 0

    def on_korean_activate(self, widget):
        self.lang = 1

    def on_preferences_activate(self, widget):
        if self.speaker == "mplayer":
            toggle = self.glade.get_widget("mplayer_button")
        else:
            toggle = self.glade.get_widget("browser_button")
        toggle.set_active(gtk.TRUE)
        self.glade.get_widget('preferences_dialog').show()

    def on_okbutton_clicked(self, widget):
        self.glade.get_widget('preferences_dialog').hide()

    def on_browser_button_toggled(self, widget):
        if widget.get_active():
            self.conf.set_string("/apps/gdick/speaker", "browser")
        else:
            self.conf.set_string("/apps/gdick/speaker", "mplayer")
    
    def on_about_activate(self, widget):
        self.glade.get_widget('about').show()

    ## about dialog bug
    #def on_about_delete_event(self, widget, *args):
    #    self.glade.get_widget('about').hide()
    #    return gtk.TRUE

    def on_combo_entry_activate(self, entry):
        self.search_wrap()
    
    def on_button_clicked(self, *args):
        self.search_wrap()

    def on_window_drag_data_received(self, widget, context, x, y, data, info, timestamp):
        if data and data.format == 8:
            self.search(data.data)
        context.finish(gtk.TRUE,gtk.FALSE, timestamp)

    def on_window_destroy(self, widget, *args):
        gtk.main_quit()

def Main():
    gtk.timeout_add(33, sleeper)
    GDick().main()
